import { Location } from '@angular/common';
import { Component, OnDestroy, OnInit } from '@angular/core';
import { ActivatedRoute, Router, UrlSegment } from '@angular/router';
import { FormBuilder, FormGroup } from '@ng-stack/forms';
import { AppService } from 'src/app/modules/shared/services/app/app.service';
import { AuthTabEnum } from '../../enums/auth-tab.enum';
import { LoginModel } from '../../models/login.model';
import { RegisterModel } from '../../models/register.model';
import { RestoreModel } from '../../models/restore.model';
import { AuthService } from '../../services/auth/auth.service';
import { ThemeSwitcherComponent } from '../theme-switcher/theme-switcher.component';

@Component({
    selector: 'app-auth',
    templateUrl: './auth.component.html',
    styleUrls: ['./auth.component.scss']
})
export class AuthComponent implements OnInit, OnDestroy {

    registerForm: FormGroup<RegisterModel>;
    loginForm: FormGroup<LoginModel>;
    restoreForm: FormGroup<RestoreModel>;
    tab: AuthTabEnum;
    selectedIndex: number;
    defaultTab = AuthTabEnum.Login;
    tabIndexes = [
        AuthTabEnum.Login,
        AuthTabEnum.Register,
        AuthTabEnum.RestorePassword,
    ];
    showRestore = false;
    AuthTabEnum = AuthTabEnum;

    constructor(
        private appService: AppService,
        private authService: AuthService,
        private activatedRoute: ActivatedRoute,
        private router: Router,
        private location: Location,
        formBuilder: FormBuilder,
    ) {
        this.registerForm = formBuilder.group<RegisterModel>({
            name: 'Name',
            surname: 'Surname',
            email: 'email@test.com',
            password: 'password',
        });
        this.loginForm = formBuilder.group<LoginModel>({
            email: 'email@test.com',
            password: 'password',
        });
        this.restoreForm = formBuilder.group<RestoreModel>({
            email: 'email@test.com',
        });

        this.handleUrl(activatedRoute.snapshot.url);
        activatedRoute.url.subscribe(url => {
            this.handleUrl(url);
        });
    }

    handleUrl(url: UrlSegment[]): void {
        const path = url[0].path;
        this.tab = path as AuthTabEnum;
        this.selectedIndex = this.getSelectedIndex();
    }

    getSelectedIndex(tab: AuthTabEnum = null): number {
        if (tab === null) {
            tab = this.tab;
        }
        let index = this.tabIndexes.findIndex(i => i === tab);
        if (index === -1) {
            index = this.tabIndexes.findIndex(i => i === this.defaultTab);
        }
        return index;
    }

    getIndexTab(): AuthTabEnum {
        return this.tabIndexes[this.selectedIndex];
    }

    selectedIndexChange(index: number): void {
        this.selectedIndex = index;
        const tab = this.getIndexTab();
        const url = this.router.createUrlTree(['..', tab], {
            relativeTo: this.activatedRoute,
        }).toString();
        this.location.go(url);
    }

    ngOnInit(): void {
        this.appService.setSidebarHidden(true);
        this.appService.addDynamicToolbarComponent(ThemeSwitcherComponent);
    }

    ngOnDestroy(): void {
        this.appService.setSidebarHidden(false);
        this.appService.removeDynamicToolbarComponent(ThemeSwitcherComponent);
    }

    register(): void {
        this.authService.createAccount(this.registerForm.getRawValue()).subscribe(data => {
            this.loginForm.patchValue({
                email: this.registerForm.controls.email.value,
                password: this.registerForm.controls.password.value,
            });
            this.selectedIndexChange(this.getSelectedIndex(AuthTabEnum.Login));
        });
    }

    login(): void {
        this.authService.login(this.loginForm.getRawValue()).subscribe(data => {
            const afterUrl = '/';
            this.router.navigateByUrl(afterUrl);
        });
    }
}
